@@ -1,129 +1,97 @@
 /*
- *   Copyright 2005 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
- *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2005 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "applet.h"
 #include "private/applet_p.h"
 
 #include "config-plasma.h"
 
-#include <cmath>
-#include <limits>
-
-#include <QFile>
-#include <QList>
 #include <QAbstractButton>
-#include <QMessageBox>
-
-#include <kactioncollection.h>
-#include <kauthorized.h>
-#include <kcolorscheme.h>
-#include <kdesktopfile.h>
+#include <QActionGroup>
 #include <QDebug>
-#include <kglobalaccel.h>
-#include <kplugininfo.h>
-#include <klocalizedstring.h>
-#include <kservice.h>
+#include <QFile>
+#include <QJSEngine>
+#include <QList>
+#include <QMessageBox>
+#include <QMetaEnum>
+
+#include <KAuthorized>
+#include <KColorScheme>
 #include <KConfigLoader>
-#include <kwindowsystem.h>
+#include <KConfigPropertyMap>
+#include <KDesktopFile>
+#include <KGlobalAccel>
+#include <KLocalizedString>
+#include <KPackage/Package>
 
 #include "containment.h"
 #include "corona.h"
-#include "package.h"
 #include "plasma.h"
-#include "scripting/appletscript.h"
 #include "pluginloader.h"
 
-#include "private/associatedapplicationmanager_p.h"
-#include "private/containment_p.h"
-#include "private/package_p.h"
 #include "debug_p.h"
+#include "private/containment_p.h"
+
+#include <cmath>
+#include <limits>
+#include <qaction.h>
 
 namespace Plasma
 {
-
-Applet::Applet(const KPluginInfo &info, QObject *parent, uint appletId)
-    :  QObject(parent),
-       d(new AppletPrivate(KService::Ptr(), &info, appletId, this))
+Applet::Applet(QObject *parentObject, const KPluginMetaData &data, const QVariantList &args)
+    : QObject(parentObject)
+    , d(new AppletPrivate(data, args.count() > 1 ? args[1].toInt() : 0, this))
 {
-    qCDebug(LOG_PLASMA) << " From KPluginInfo, valid? " << info.isValid();
-    // WARNING: do not access config() OR globalConfig() in this method!
-    //          that requires a scene, which is not available at this point
-    d->init();
-    d->setupPackage();
-}
+    if (!args.isEmpty()) {
+        const QVariant first = args.first();
+        if (first.canConvert<KPackage::Package>()) {
+            d->package = first.value<KPackage::Package>();
+        }
+    }
+    d->icon = d->appletDescription.iconName();
 
-Applet::Applet(QObject *parent, const QString &serviceID, uint appletId)
-    :  QObject(parent),
-       d(new AppletPrivate(KService::serviceByStorageId(serviceID), 0, appletId, this))
-{
-    // WARNING: do not access config() OR globalConfig() in this method!
-    //          that requires a scene, which is not available at this point
-    d->init();
-    d->setupPackage();
-}
-
-Applet::Applet(QObject *parentObject, const QVariantList &args)
-    :  QObject(0),
-       d(new AppletPrivate(
-             KService::serviceByStorageId(args.count() > 0 && args.first().canConvert<QString>() ? args[0].toString() : QString()), 0,
-             args.count() > 1 ? args[1].toInt() : 0, this))
-{
-    setParent(parentObject);
-    if (args.count() > 0 && args.first().canConvert<QVariantMap>()) {
-        d->appletDescription = KPluginInfo(args);
+    if (args.contains(QVariant::fromValue(QStringLiteral("org.kde.plasma:force-create")))) {
+        setProperty("org.kde.plasma:force-create", true);
     }
 
     // WARNING: do not access config() OR globalConfig() in this method!
     //          that requires a scene, which is not available at this point
-    d->init(QString(), args.mid(2));
-    d->setupPackage();
-}
-
-Applet::Applet(const QString &packagePath, uint appletId)
-    : QObject(0),
-      d(new AppletPrivate(KService::Ptr(new KService(packagePath + "/metadata.desktop")), 0, appletId, this))
-{
-    d->init(packagePath);
+    d->init(args.mid(2));
     d->setupPackage();
 }
 
 Applet::~Applet()
 {
+    for (QAction *a : d->actions.values()) {
+        disconnect(a, nullptr, this, nullptr);
+    }
+    for (QAction *a : d->contextualActions) {
+        disconnect(a, nullptr, this, nullptr);
+    }
     if (d->transient) {
         d->resetConfigurationObject();
     }
-    //let people know that i will die
-    emit appletDeleted(this);
+    // let people know that i will die
+    Q_EMIT appletDeleted(this);
 
+    // ConfigLoader is deleted when AppletPrivate closes not Applet
+    // It saves on closure and emits a signal.
+    // disconnect early to avoid a crash. See  411221
+    if (d->configLoader) {
+        disconnect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
+    }
     delete d;
 }
 
 void Applet::init()
 {
-    if (d->script) {
-        if (!d->script->init() && !d->failed) {
-            setLaunchErrorMessage(i18n("Script initialization failed"));
-        }
-    }
+    // Don't implement anything here, it will be overridden by subclasses
 }
 
 uint Applet::id() const
@@ -131,6 +99,11 @@ uint Applet::id() const
     return d->appletId;
 }
 
+QVariantList Applet::startupArguments() const
+{
+    return d->startupArguments;
+}
+
 void Applet::save(KConfigGroup &g) const
 {
     if (d->transient || !d->appletDescription.isValid()) {
@@ -142,17 +115,17 @@ void Applet::save(KConfigGroup &g) const
         group = *d->mainConfigGroup();
     }
 
-    //qCDebug(LOG_PLASMA) << "saving" << pluginName() << "to" << group.name();
+    // qCDebug(LOG_PLASMA) << "saving" << pluginName() << "to" << group.name();
     // we call the dptr member directly for locked since isImmutable()
     // also checks kiosk and parent containers
     group.writeEntry("immutability", (int)d->immutability);
-    group.writeEntry("plugin", d->appletDescription.pluginName());
+    group.writeEntry("plugin", d->appletDescription.pluginId());
 
     if (!d->started) {
         return;
     }
 
-    KConfigGroup appletConfigGroup(&group, "Configuration");
+    KConfigGroup appletConfigGroup(&group, QStringLiteral("Configuration"));
     saveState(appletConfigGroup);
 
     if (d->configLoader) {
@@ -166,10 +139,9 @@ void Applet::save(KConfigGroup &g) const
 
 void Applet::restore(KConfigGroup &group)
 {
-
     setImmutability((Types::ImmutabilityType)group.readEntry("immutability", (int)Types::Mutable));
 
-    KConfigGroup shortcutConfig(&group, "Shortcuts");
+    KConfigGroup shortcutConfig(&group, QStringLiteral("Shortcuts"));
     QString shortcutText = shortcutConfig.readEntryUntranslated("global", QString());
     if (!shortcutText.isEmpty()) {
         setGlobalShortcut(QKeySequence(shortcutText));
@@ -184,18 +156,20 @@ void Applet::restore(KConfigGroup &group)
                  */
     }
 
-    // local shortcut, if any
-    //TODO: implement; the shortcut will need to be registered with the containment
-    /*
-    #include "accessmanager.h"
-    #include "private/plasmoidservice_p.h"
-    #include "authorizationmanager.h"
-    #include "authorizationmanager.h"
-    shortcutText = shortcutConfig.readEntryUntranslated("local", QString());
-    if (!shortcutText.isEmpty()) {
-        //TODO: implement; the shortcut
+    // User background hints
+    // TODO support flags in the config
+    QByteArray hintsString = config().readEntry("UserBackgroundHints", QString()).toUtf8();
+    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
+    bool ok;
+    int value = hintEnum.keyToValue(hintsString.constData(), &ok);
+    if (ok) {
+        d->userBackgroundHints = Plasma::Types::BackgroundHints(value);
+        d->userBackgroundHintsInitialized = true;
+        Q_EMIT userBackgroundHintsChanged();
+        if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
+            Q_EMIT effectiveBackgroundHintsChanged();
+        }
     }
-    */
 }
 
 void Applet::setLaunchErrorMessage(const QString &message)
@@ -210,10 +184,6 @@ void Applet::setLaunchErrorMessage(const QString &message)
 
 void Applet::saveState(KConfigGroup &group) const
 {
-    if (d->script) {
-        emit d->script->saveState(group);
-    }
-
     if (group.config()->name() != config().config()->name()) {
         // we're being saved to a different file!
         // let's just copy the current values in our configuration over
@@ -225,23 +195,23 @@ void Applet::saveState(KConfigGroup &group) const
 KConfigGroup Applet::config() const
 {
     if (d->transient) {
-        return KConfigGroup(KSharedConfig::openConfig(), "PlasmaTransientsConfig");
+        return KConfigGroup(KSharedConfig::openConfig(), QStringLiteral("PlasmaTransientsConfig"));
     }
 
     if (isContainment()) {
         return *(d->mainConfigGroup());
     }
 
-    return KConfigGroup(d->mainConfigGroup(), "Configuration");
+    return KConfigGroup(d->mainConfigGroup(), QStringLiteral("Configuration"));
 }
 
 KConfigGroup Applet::globalConfig() const
 {
     KConfigGroup globalAppletConfig;
-    QString group = isContainment() ? "ContainmentGlobals" : "AppletGlobals";
+    QString group = isContainment() ? QStringLiteral("ContainmentGlobals") : QStringLiteral("AppletGlobals");
 
     Containment *cont = containment();
-    Corona *corona = 0;
+    Corona *corona = nullptr;
     if (cont) {
         corona = cont->corona();
     }
@@ -258,12 +228,11 @@ KConfigGroup Applet::globalConfig() const
 void Applet::destroy()
 {
     if (immutability() != Types::Mutable || d->transient || !d->started) {
-        return; //don't double delete
+        return; // don't double delete
     }
 
-    d->transient = true;
-    emit destroyedChanged(true);
-    //FIXME: an animation on leave if !isContainment() would be good again .. which should be handled by the containment class
+    d->setDestroyed(true);
+    // FIXME: an animation on leave if !isContainment() would be good again .. which should be handled by the containment class
     d->cleanUpAndDelete();
 }
 
@@ -275,10 +244,10 @@ bool Applet::destroyed() const
 KConfigLoader *Applet::configScheme() const
 {
     if (!d->configLoader) {
-        const QString xmlPath = d->package ? d->package->filePath("mainconfigxml") : QString();
+        const QString xmlPath = d->package.isValid() ? d->package.filePath("mainconfigxml") : QString();
         KConfigGroup cfg = config();
         if (xmlPath.isEmpty()) {
-            d->configLoader = new KConfigLoader(cfg, 0);
+            d->configLoader = new KConfigLoader(cfg, nullptr);
         } else {
             QFile file(xmlPath);
             d->configLoader = new KConfigLoader(cfg, &file);
@@ -289,37 +258,33 @@ KConfigLoader *Applet::configScheme() const
     return d->configLoader;
 }
 
-Package Applet::package() const
+KConfigPropertyMap *Applet::configuration()
 {
-    Package p;
-    if (d->package) {
-        p.d->internalPackage = new KPackage::Package(*d->package);
+    if (!d->configPropertyMap) {
+        d->configPropertyMap = new KConfigPropertyMap(configScheme(), this);
     }
-    return p;
+    return d->configPropertyMap;
 }
 
 KPackage::Package Applet::kPackage() const
 {
-    return d->package ? *d->package : KPackage::Package();
+    return d->package;
 }
 
-void Applet::updateConstraints(Plasma::Types::Constraints constraints)
+void Applet::updateConstraints(Constraints constraints)
 {
     d->scheduleConstraintsUpdate(constraints);
 }
 
-void Applet::constraintsEvent(Plasma::Types::Constraints constraints)
+void Applet::constraintsEvent(Constraints constraints)
 {
-    //NOTE: do NOT put any code in here that reacts to constraints updates
+    // NOTE: do NOT put any code in here that reacts to constraints updates
     //      as it will not get called for any applet that reimplements constraintsEvent
     //      without calling the Applet:: version as well, which it shouldn't need to.
     //      INSTEAD put such code into flushPendingConstraintsEvents
     Q_UNUSED(constraints)
-    //qCDebug(LOG_PLASMA) << constraints << "constraints are FormFactor: " << formFactor()
+    // qCDebug(LOG_PLASMA) << constraints << "constraints are FormFactor: " << formFactor()
     //         << ", Location: " << location();
-    if (d->script) {
-        d->script->constraintsEvent(constraints);
-    }
 }
 
 QString Applet::title() const
@@ -335,9 +300,14 @@ QString Applet::title() const
     return i18n("Unknown");
 }
 
-void Applet::setTitle(const QString &title) const
+void Applet::setTitle(const QString &title)
 {
+    if (title == d->customTitle) {
+        return;
+    }
+
     d->customTitle = title;
+    Q_EMIT titleChanged(title);
 }
 
 QString Applet::icon() const
@@ -347,14 +317,95 @@ QString Applet::icon() const
 
 void Applet::setIcon(const QString &icon)
 {
+    if (icon == d->icon) {
+        return;
+    }
+
     d->icon = icon;
+    Q_EMIT iconChanged(icon);
 }
 
-KPluginInfo Applet::pluginInfo() const
+bool Applet::isBusy() const
+{
+    return d->busy;
+}
+
+void Applet::setBusy(bool busy)
+{
+    if (busy == d->busy) {
+        return;
+    }
+
+    d->busy = busy;
+    Q_EMIT busyChanged(busy);
+}
+
+Plasma::Types::BackgroundHints Applet::backgroundHints() const
+{
+    return d->backgroundHints;
+}
+
+void Applet::setBackgroundHints(Plasma::Types::BackgroundHints hint)
+{
+    if (d->backgroundHints == hint) {
+        return;
+    }
+
+    Plasma::Types::BackgroundHints oldeffectiveHints = effectiveBackgroundHints();
+
+    d->backgroundHints = hint;
+    Q_EMIT backgroundHintsChanged();
+
+    if (oldeffectiveHints != effectiveBackgroundHints()) {
+        Q_EMIT effectiveBackgroundHintsChanged();
+    }
+}
+
+Plasma::Types::BackgroundHints Applet::effectiveBackgroundHints() const
+{
+    if (d->userBackgroundHintsInitialized && (d->backgroundHints & Plasma::Types::ConfigurableBackground)) {
+        return d->userBackgroundHints;
+    } else {
+        return d->backgroundHints;
+    }
+}
+
+Plasma::Types::BackgroundHints Applet::userBackgroundHints() const
+{
+    return d->userBackgroundHints;
+}
+
+void Applet::setUserBackgroundHints(Plasma::Types::BackgroundHints hint)
+{
+    if (d->userBackgroundHints == hint && d->userBackgroundHintsInitialized) {
+        return;
+    }
+
+    d->userBackgroundHints = hint;
+    d->userBackgroundHintsInitialized = true;
+    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
+    config().writeEntry("UserBackgroundHints", hintEnum.valueToKey(d->userBackgroundHints));
+    if (containment() && containment()->corona()) {
+        containment()->corona()->requestConfigSync();
+    }
+
+    Q_EMIT userBackgroundHintsChanged();
+
+    if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
+        Q_EMIT effectiveBackgroundHintsChanged();
+    }
+}
+
+KPluginMetaData Applet::pluginMetaData() const
 {
     return d->appletDescription;
 }
 
+QString Applet::pluginName() const
+{
+    return d->appletDescription.isValid() ? d->appletDescription.pluginId() : QString();
+}
+
 Types::ImmutabilityType Applet::immutability() const
 {
     // if this object is itself system immutable, then just return that; it's the most
@@ -364,7 +415,7 @@ Types::ImmutabilityType Applet::immutability() const
         return Types::SystemImmutable;
     }
 
-    //Returning the more strict immutability between the applet immutability, Containment and Corona
+    // Returning the more strict immutability between the applet immutability, Containment and Corona
     Types::ImmutabilityType upperImmutability = Types::Mutable;
 
     if (isContainment()) {
@@ -403,7 +454,12 @@ void Applet::setImmutability(const Types::ImmutabilityType immutable)
     }
 
     d->immutability = immutable;
-    updateConstraints(Types::ImmutableConstraint);
+    updateConstraints(ImmutableConstraint);
+}
+
+bool Applet::immutable() const
+{
+    return immutability() != Types::Mutable;
 }
 
 QString Applet::launchErrorMessage() const
@@ -421,14 +477,36 @@ bool Applet::configurationRequired() const
     return d->needsConfig;
 }
 
+QString Applet::configurationRequiredReason() const
+{
+    return d->configurationRequiredReason;
+}
+
 void Applet::setConfigurationRequired(bool needsConfig, const QString &reason)
 {
-    if (d->needsConfig == needsConfig) {
+    if (d->needsConfig == needsConfig && reason == d->configurationRequiredReason) {
         return;
     }
 
     d->needsConfig = needsConfig;
-    d->showConfigurationRequiredMessage(needsConfig, reason);
+    d->configurationRequiredReason = reason;
+
+    Q_EMIT configurationRequiredChanged(needsConfig, reason);
+}
+
+void Applet::setConstraintHints(ConstraintHints constraintHints)
+{
+    if (d->constraintHints == constraintHints) {
+        return;
+    }
+
+    d->constraintHints = constraintHints;
+    Q_EMIT constraintHintsChanged(constraintHints);
+}
+
+Applet::ConstraintHints Applet::constraintHints() const
+{
+    return d->constraintHints;
 }
 
 bool Applet::isUserConfiguring() const
@@ -443,7 +521,7 @@ void Applet::setUserConfiguring(bool configuring)
     }
 
     d->userConfiguring = configuring;
-    emit userConfiguringChanged(configuring);
+    Q_EMIT userConfiguringChanged(configuring);
 }
 
 Types::ItemStatus Applet::status() const
@@ -457,12 +535,12 @@ void Applet::setStatus(const Types::ItemStatus status)
         return;
     }
     d->itemStatus = status;
-    emit statusChanged(status);
+    Q_EMIT statusChanged(status);
 }
 
 void Applet::flushPendingConstraintsEvents()
 {
-    if (d->pendingConstraints == Types::NoConstraint) {
+    if (d->pendingConstraints == NoConstraint) {
         return;
     }
 
@@ -470,72 +548,58 @@ void Applet::flushPendingConstraintsEvents()
         d->constraintsTimer.stop();
     }
 
-    //qCDebug(LOG_PLASMA) << "fushing constraints: " << d->pendingConstraints << "!!!!!!!!!!!!!!!!!!!!!!!!!!!";
-    Plasma::Types::Constraints c = d->pendingConstraints;
-    d->pendingConstraints = Types::NoConstraint;
+    // qCDebug(LOG_PLASMA) << "flushing constraints: " << d->pendingConstraints << "!!!!!!!!!!!!!!!!!!!!!!!!!!!";
+    Constraints c = d->pendingConstraints;
+    d->pendingConstraints = NoConstraint;
 
-    if (c & Plasma::Types::UiReadyConstraint) {
+    if (c & UiReadyConstraint) {
         d->setUiReady();
     }
 
-    if (c & Plasma::Types::StartupCompletedConstraint) {
-        //common actions
+    if (c & StartupCompletedConstraint) {
+        // common actions
         bool unlocked = immutability() == Types::Mutable;
-        QAction *closeApplet = d->actions->action("remove");
+        QAction *closeApplet = d->actions.value(QStringLiteral("remove"));
         if (closeApplet) {
             closeApplet->setEnabled(unlocked);
             closeApplet->setVisible(unlocked);
             connect(closeApplet, SIGNAL(triggered(bool)), this, SLOT(askDestroy()), Qt::UniqueConnection);
         }
 
-        QAction *configAction = d->actions->action("configure");
+        QAction *configAction = d->actions.value(QStringLiteral("configure"));
         if (configAction) {
-
             if (d->hasConfigurationInterface) {
-                bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
+                bool canConfig = unlocked || KAuthorized::authorize(QStringLiteral("plasma/allow_configure_when_locked"));
                 configAction->setVisible(canConfig);
                 configAction->setEnabled(canConfig);
             }
         }
-
-        QAction *runAssociatedApplication = d->actions->action("run associated application");
-        if (runAssociatedApplication) {
-            connect(runAssociatedApplication, SIGNAL(triggered(bool)), this, SLOT(runAssociatedApplication()), Qt::UniqueConnection);
-        }
-
-        d->updateShortcuts();
     }
 
-    if (c & Plasma::Types::ImmutableConstraint) {
+    if (c & ImmutableConstraint) {
         bool unlocked = immutability() == Types::Mutable;
-        QAction *action = d->actions->action("remove");
+        QAction *action = d->actions.value(QStringLiteral("remove"));
         if (action) {
             action->setVisible(unlocked);
             action->setEnabled(unlocked);
         }
 
-        action = d->actions->action("alternatives");
-        if (action) {
-            action->setVisible(unlocked);
-            action->setEnabled(unlocked);
-        }
-
-        action = d->actions->action("configure");
+        action = d->actions.value(QStringLiteral("configure"));
         if (action && d->hasConfigurationInterface) {
-            bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
+            bool canConfig = unlocked || KAuthorized::authorize(QStringLiteral("plasma/allow_configure_when_locked"));
             action->setVisible(canConfig);
             action->setEnabled(canConfig);
         }
 
-        //an immutable constraint will alwasy happen at startup
-        //make sure don't emit a change signal for nothing
+        // an immutable constraint will always happen at startup
+        // make sure don't emit a change signal for nothing
         if (d->oldImmutability != immutability()) {
-            emit immutabilityChanged(immutability());
+            Q_EMIT immutabilityChanged(immutability());
         }
         d->oldImmutability = immutability();
     }
 
-    // now take care of constraints in special subclass: Contaiment
+    // now take care of constraints in special subclass: Containment
     Containment *containment = qobject_cast<Plasma::Containment *>(this);
     if (containment) {
         containment->d->containmentConstraintsEvent(c);
@@ -544,7 +608,7 @@ void Applet::flushPendingConstraintsEvents()
     // pass the constraint on to the actual subclass
     constraintsEvent(c);
 
-    if (c & Types::StartupCompletedConstraint) {
+    if (c & StartupCompletedConstraint) {
         // start up is done, we can now go do a mod timer
         if (d->modificationsTimer) {
             if (d->modificationsTimer->isActive()) {
@@ -555,24 +619,76 @@ void Applet::flushPendingConstraintsEvents()
         }
     }
 
-    if (c & Plasma::Types::FormFactorConstraint) {
-        emit formFactorChanged(formFactor());
+    if (c & FormFactorConstraint) {
+        Q_EMIT formFactorChanged(formFactor());
     }
 
-    if (c & Plasma::Types::LocationConstraint) {
-        emit locationChanged(location());
+    if (c & LocationConstraint) {
+        Q_EMIT locationChanged(location());
     }
 }
 
 QList<QAction *> Applet::contextualActions()
 {
-    //qCDebug(LOG_PLASMA) << "empty context actions";
-    return d->script ? d->script->contextualActions() : QList<QAction *>();
+    return d->contextualActions;
 }
 
-KActionCollection *Applet::actions() const
+QQmlListProperty<QAction> Applet::qmlContextualActions()
 {
-    return d->actions;
+    return QQmlListProperty<QAction>(this,
+                                     nullptr,
+                                     AppletPrivate::contextualActions_append,
+                                     AppletPrivate::contextualActions_count,
+                                     AppletPrivate::contextualActions_at,
+                                     AppletPrivate::contextualActions_clear,
+                                     AppletPrivate::contextualActions_replace,
+                                     AppletPrivate::contextualActions_removeLast);
+}
+
+void Applet::setInternalAction(const QString &name, QAction *action)
+{
+    if (name.isEmpty()) {
+        return;
+    }
+
+    action->setObjectName(name);
+    QAction *oldAction = d->actions.value(name);
+    if (oldAction && QJSEngine::objectOwnership(oldAction) == QJSEngine::CppOwnership) {
+        delete oldAction;
+    }
+
+    d->actions[name] = action;
+
+    QObject::connect(action, &QObject::destroyed, this, [this, name]() {
+        d->actions.remove(name);
+        Q_EMIT internalActionsChanged(d->actions.values());
+    });
+
+    Q_EMIT internalActionsChanged(d->actions.values());
+}
+
+QAction *Applet::internalAction(const QString &name) const
+{
+    return d->actions.value(name);
+}
+
+void Applet::removeInternalAction(const QString &name)
+{
+    QAction *action = d->actions.value(name);
+
+    if (action && QJSEngine::objectOwnership(action) == QJSEngine::CppOwnership) {
+        disconnect(action, &QObject::destroyed, this, nullptr); // Avoid emitting signal again
+        delete action;
+    }
+
+    d->actions.remove(name);
+
+    Q_EMIT internalActionsChanged(d->actions.values());
+}
+
+QList<QAction *> Applet::internalActions() const
+{
+    return d->actions.values();
 }
 
 Types::FormFactor Applet::formFactor() const
@@ -580,7 +696,7 @@ Types::FormFactor Applet::formFactor() const
     Containment *c = containment();
     QObject *pw = qobject_cast<QObject *>(parent());
     Plasma::Applet *parentApplet = qobject_cast<Plasma::Applet *>(pw);
-    //assumption: this loop is usually is -really- short or doesn't run at all
+    // assumption: this loop is usually is -really- short or doesn't run at all
     while (!parentApplet && pw && pw->parent()) {
         pw = pw->parent();
         parentApplet = qobject_cast<Plasma::Applet *>(pw);
@@ -589,13 +705,20 @@ Types::FormFactor Applet::formFactor() const
     return c ? c->d->formFactor : Plasma::Types::Planar;
 }
 
+Types::ContainmentDisplayHints Applet::containmentDisplayHints() const
+{
+    Containment *c = containment();
+
+    return c ? c->d->containmentDisplayHints : Plasma::Types::NoContainmentDisplayHint;
+}
+
 Containment *Applet::containment() const
 {
     Containment *c = qobject_cast<Containment *>(const_cast<Applet *>(this));
     if (c && c->isContainment()) {
         return c;
     } else {
-        c = 0;
+        c = nullptr;
     }
 
     QObject *parent = this->parent();
@@ -619,19 +742,24 @@ void Applet::setGlobalShortcut(const QKeySequence &shortcut)
         d->activationAction = new QAction(this);
         d->activationAction->setText(i18n("Activate %1 Widget", title()));
         d->activationAction->setObjectName(QStringLiteral("activate widget %1").arg(id())); // NO I18N
-        connect(d->activationAction, SIGNAL(triggered()), this, SIGNAL(activated()));
-        connect(d->activationAction, SIGNAL(changed()),
-                this, SLOT(globalShortcutChanged()));
+        connect(d->activationAction, &QAction::triggered, this, &Applet::activated);
+        connect(KGlobalAccel::self(), &KGlobalAccel::globalShortcutChanged, this, [this](QAction *action, const QKeySequence &shortcut) {
+            if (action == d->activationAction) {
+                d->activationAction->setShortcut(shortcut);
+                d->globalShortcutChanged();
+            }
+        });
     } else if (d->activationAction->shortcut() == shortcut) {
         return;
     }
 
     d->activationAction->setShortcut(shortcut);
     d->globalShortcutEnabled = true;
-    QList<QKeySequence> seqs;
-    seqs << shortcut;
+    QList<QKeySequence> seqs{shortcut};
     KGlobalAccel::self()->setShortcut(d->activationAction, seqs, KGlobalAccel::NoAutoloading);
     d->globalShortcutChanged();
+
+    Q_EMIT globalShortcutChanged(shortcut);
 }
 
 QKeySequence Applet::globalShortcut() const
@@ -663,7 +791,7 @@ void Applet::setHasConfigurationInterface(bool hasInterface)
         return;
     }
 
-    QAction *configAction = d->actions->action(QStringLiteral("configure"));
+    QAction *configAction = d->actions.value(QStringLiteral("configure"));
     if (configAction) {
         bool enable = hasInterface;
         if (enable) {
@@ -678,73 +806,17 @@ void Applet::setHasConfigurationInterface(bool hasInterface)
 
 void Applet::configChanged()
 {
-    if (d->script) {
-        if (d->configLoader) {
-            d->configLoader->load();
-        }
-        d->script->configChanged();
+    if (d->configLoader) {
+        d->configLoader->load();
     }
 }
 
-void Applet::setAssociatedApplication(const QString &string)
+QString Applet::filePath(const QByteArray &key, const QString &filename) const
 {
-    AssociatedApplicationManager::self()->setApplication(this, string);
-
-    QAction *runAssociatedApplication = d->actions->action("run associated application");
-    if (runAssociatedApplication) {
-        bool valid = AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
-        runAssociatedApplication->setVisible(valid);
-        runAssociatedApplication->setEnabled(valid);
+    if (d->package.isValid()) {
+        return d->package.filePath(key, filename);
     }
-}
-
-void Applet::setAssociatedApplicationUrls(const QList<QUrl> &urls)
-{
-    AssociatedApplicationManager::self()->setUrls(this, urls);
-
-    QAction *runAssociatedApplication = d->actions->action("run associated application");
-    if (runAssociatedApplication) {
-        bool valid = AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
-        runAssociatedApplication->setVisible(valid);
-        runAssociatedApplication->setEnabled(valid);
-    }
-}
-
-QString Applet::associatedApplication() const
-{
-    return AssociatedApplicationManager::self()->application(this);
-}
-
-QList<QUrl> Applet::associatedApplicationUrls() const
-{
-    return AssociatedApplicationManager::self()->urls(this);
-}
-
-void Applet::runAssociatedApplication()
-{
-    AssociatedApplicationManager::self()->run(this);
-}
-
-bool Applet::hasValidAssociatedApplication() const
-{
-    return AssociatedApplicationManager::self()->appletHasValidAssociatedApplication(this);
-}
-
-Applet *Applet::loadPlasmoid(const QString &path, uint appletId)
-{
-    const QString metadataPath = path + QLatin1String("/metadata.desktop");
-    if (QFile::exists(metadataPath)) {
-        KService service(metadataPath);
-        const QStringList &types = service.serviceTypes();
-
-        if (types.contains(QStringLiteral("Plasma/Containment"))) {
-            return new Containment(path, appletId);
-        } else {
-            return new Applet(path, appletId);
-        }
-    }
-
-    return 0;
+    return QString();
 }
 
 void Applet::timerEvent(QTimerEvent *event)
@@ -762,7 +834,7 @@ void Applet::timerEvent(QTimerEvent *event)
 
         // Don't flushPendingConstraints if we're just starting up
         // flushPendingConstraints will be called by Corona
-        if (!(d->pendingConstraints & Plasma::Types::StartupCompletedConstraint)) {
+        if (!(d->pendingConstraints & StartupCompletedConstraint)) {
             flushPendingConstraintsEvents();
         }
     } else if (d->modificationsTimer && event->timerId() == d->modificationsTimer->timerId()) {
@@ -771,19 +843,19 @@ void Applet::timerEvent(QTimerEvent *event)
         KConfigGroup cg;
 
         save(cg);
-        emit configNeedsSaving();
+        Q_EMIT configNeedsSaving();
     }
 }
 
 bool Applet::isContainment() const
 {
-    //HACK: this is a special case for the systray
-    //containment in an applet that is not a containment
+    // HACK: this is a special case for the systray
+    // containment in an applet that is not a containment
     Applet *pa = qobject_cast<Applet *>(parent());
     if (pa && !pa->isContainment()) {
         return true;
     }
-    //normal "acting as a containment" condition
+    // normal "acting as a containment" condition
     return qobject_cast<const Containment *>(this) && qobject_cast<Corona *>(parent());
 }
 
