@@ -1,35 +1,23 @@
 /*
- *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
- *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_CONTAINMENT_H
 #define PLASMA_CONTAINMENT_H
 
-#include <ksharedconfig.h>
+#include <KPluginMetaData>
+#include <KSharedConfig>
 #include <plasma/applet.h>
+#include <qtmetamacros.h>
+
+class QQuickItem;
 
 namespace Plasma
 {
-
-class DataEngine;
-class Package;
 class Corona;
 class ContainmentActions;
 class ContainmentPrivate;
@@ -42,7 +30,7 @@ class ContainmentPrivate;
  * Containment objects provide the means to group applets into functional sets.
  * They also provide the following:
  *
- * creation of focussing event
+ * creation of focusing event
  * - drawing of the background image (which can be interactive)
  * - form factors (e.g. panel, desktop, full screen, etc)
  * - applet layout management
@@ -58,44 +46,105 @@ class ContainmentPrivate;
 class PLASMA_EXPORT Containment : public Applet
 {
     Q_OBJECT
-    Q_PROPERTY(QString wallpaper READ wallpaper WRITE setWallpaper NOTIFY wallpaperChanged)
+
+    /**
+     * List of applets this containment has: the containments
+     * KF6: this should be AppletQuickItem *
+     */
+    Q_PROPERTY(QList<Plasma::Applet *> applets READ applets NOTIFY appletsChanged)
+
+    /**
+     * The corona for this contaiment
+     */
+    Q_PROPERTY(Plasma::Corona *corona READ corona CONSTANT)
+
+    /**
+     * Type of this containment
+     */
+    Q_PROPERTY(Plasma::Containment::Type containmentType READ containmentType NOTIFY containmentTypeChanged)
+
+    /**
+     * Activity UID of this containment
+     */
+    Q_PROPERTY(QString activity READ activity NOTIFY activityChanged)
+
+    /**
+     * Activity name of this containment
+     */
+    Q_PROPERTY(QString activityName READ activityName NOTIFY activityNameChanged)
+
+    Q_PROPERTY(Plasma::Types::ContainmentDisplayHints containmentDisplayHints READ containmentDisplayHints WRITE setContainmentDisplayHints NOTIFY
+                   containmentDisplayHintsChanged)
+
+    Q_PROPERTY(QString wallpaperPlugin READ wallpaperPlugin WRITE setWallpaperPlugin NOTIFY wallpaperPluginChanged)
+    Q_PROPERTY(QObject *wallpaperGraphicsObject READ wallpaperGraphicsObject WRITE setWallpaperGraphicsObject NOTIFY wallpaperGraphicsObjectChanged)
+
     Q_PROPERTY(bool isUiReady READ isUiReady NOTIFY uiReadyChanged)
 
+    /**
+     * The screen number this containment is serving as the desktop for, or -1 if none
+     */
+    Q_PROPERTY(int screen READ screen NOTIFY screenChanged)
+
+    /**
+     * screen area free of panels: the coordinates are relative to the containment,
+     * it's independent from the screen position
+     * For more precise available geometry use availableScreenRegion()
+     */
+    Q_PROPERTY(QRectF availableScreenRect READ availableRelativeScreenRect NOTIFY availableRelativeScreenRectChanged)
+
+    /**
+     * The available region of this screen, panels excluded. It's a list of rectangles
+     */
+    Q_PROPERTY(QList<QRectF> availableScreenRegion READ availableRelativeScreenRegion NOTIFY availableRelativeScreenRegionChanged)
+
+    /**
+     * Provides access to the geometry of the applet is in.
+     * Can be useful to figure out what's the absolute position of the applet.
+     */
+    Q_PROPERTY(QRectF screenGeometry READ screenGeometry NOTIFY screenGeometryChanged)
+
 public:
     /**
-     * @param parent the QObject this applet is parented to
-     * @param serviceId the name of the .desktop file containing the
-     *      information about the widget
-     * @param containmentId a unique id used to differentiate between multiple
-     *      instances of the same Applet type
-     */
-    explicit Containment(QObject *parent = 0,
-                         const QString &serviceId = QString(),
-                         uint containmentId = 0);
-
-    /**
-     * This constructor is to be used with the plugin loading systems
-     * found in KPluginInfo and KService. The argument list is expected
-     * to have two elements: the KService service ID for the desktop entry
-     * and an applet ID which must be a base 10 number.
+     * This constructor can be used with the KCoreAddons plugin loading system.
+     * The argument list is expected to have contain the KPackage of the applet,
+     * the meta data file path (for compatibility) and an applet ID which must be a base 10 number.
      *
      * @param parent a QObject parent; you probably want to pass in 0
-     * @param args a list of strings containing two entries: the service id
-     *      and the applet id
+     * @param data, KPluginMetaData used to create this plugin
+     * @param args a list of strings containing the applet id
+     * @since 5.86
      */
-    Containment(QObject *parent, const QVariantList &args);
+    explicit Containment(QObject *parentObject, const KPluginMetaData &data, const QVariantList &args);
 
-    ~Containment();
+    ~Containment() override;
 
     /**
      * Reimplemented from Applet
      */
-    void init() Q_DECL_OVERRIDE;
+    void init() override;
+
+    /**
+     * This enumeration describes the type of the Containment.
+     * DesktopContainments represent main containments that will own a screen in a mutually exclusive fashion,
+     * while PanelContainments are accessories which can be present multiple per screen.
+     *
+     * This value is specified in the "X-Plasma-ContainmentType" JSON-metadata value of containments.
+     */
+    enum Type {
+        NoContainment = -1, /**< @internal */
+        Desktop = 0, /**< A desktop containment */
+        Panel, /**< A desktop panel */
+        Custom = 127, /**< A containment that is neither a desktop nor a panel but something application specific */
+        CustomPanel = 128, /**< A customized desktop panel. "CustomPanel" in metadata */
+        CustomEmbedded = 129, /**< A customized containment embedded in another applet */
+    };
+    Q_ENUM(Type)
 
     /**
      * Returns the type of containment
      */
-    Plasma::Types::ContainmentType containmentType() const;
+    Type containmentType() const;
 
     /**
      * Returns the Corona (if any) that this Containment is hosted by
@@ -108,20 +157,23 @@ public:
      * @param name the plugin name for the applet, as given by
      *        KPluginInfo::pluginName()
      * @param args argument list to pass to the plasmoid
-     * @param geometry where to place the applet, or to auto-place it if an invalid
-     *                 is provided
+     * @param geometryHint an hint to pass to the GUI on the location
+     *           and size we prefer for the newly created applet;
+     *           the gui might choose whether to respect or not this hint
      *
      * @return a pointer to the applet on success, or 0 on failure
      */
-    Applet *createApplet(const QString &name, const QVariantList &args = QVariantList());
+    Applet *createApplet(const QString &name, const QVariantList &args = QVariantList(), const QRectF &geometryHint = QRectF());
 
     /**
      * Add an existing applet to this Containment
      *
      * @param applet the applet that should be added
-     * @param pos the containment-relative position
+     * @param geometryHint an hint to pass to the GUI on the location
+     *           and size we prefer for the newly created applet;
+     *           the gui might choose whether to respect or not this hint
      */
-    void addApplet(Applet *applet);
+    Q_INVOKABLE void addApplet(Applet *applet, const QRectF &geometryHint = QRectF());
 
     /**
      * @return the applets currently in this Containment
@@ -131,6 +183,7 @@ public:
     /**
      * @return the screen number this containment is serving as the desktop for
      *         or -1 if none
+     * TODO KF6 virtual? this shouldbe available to applet as well
      */
     int screen() const;
 
@@ -145,25 +198,25 @@ public:
      * @reimp
      * @sa Applet::save(KConfigGroup &)
      */
-    void save(KConfigGroup &group) const Q_DECL_OVERRIDE;
+    void save(KConfigGroup &group) const override;
 
     /**
      * @reimp
      * @sa Applet::restore(KConfigGroup &)
      */
-    void restore(KConfigGroup &group) Q_DECL_OVERRIDE;
+    void restore(KConfigGroup &group) override;
 
     /**
      * Sets wallpaper plugin.
      *
      * @param pluginName the name of the wallpaper to attempt to load
      */
-    void setWallpaper(const QString &pluginName);
+    void setWallpaperPlugin(const QString &pluginName);
 
     /**
      * Return wallpaper plugin.
      */
-    QString wallpaper() const;
+    QString wallpaperPlugin() const;
 
     /**
      * Sets the current activity by id
@@ -174,9 +227,16 @@ public:
 
     /**
      * @return the current activity id associated with this containment
+     * TODO KF6: this should be available to Appelt as well as a property... virtual?
      */
     QString activity() const;
 
+    /**
+     * @return Activity name corresponding to the activity UID
+     * @see activity
+     */
+    QString activityName() const;
+
     /**
      * Sets a containmentactions plugin.
      *
@@ -197,6 +257,22 @@ public:
      */
     bool isUiReady() const;
 
+    /**
+     * @returns The available screen rect (excluding panels) for the screen this containment is associated to,
+     * empty rectangle if the containment is not active in a screen
+     */
+    QRectF availableRelativeScreenRect() const;
+
+    /**
+     * @returns The available region of this screen, panels excluded. It's a list of rectangles
+     */
+    QList<QRectF> availableRelativeScreenRegion() const;
+
+    /**
+     * @returns The geometry of the screen this containment is associated to
+     */
+    QRectF screenGeometry() const;
+
 Q_SIGNALS:
     /**
      * This signal is emitted when a new applet is added in the containment
@@ -204,14 +280,36 @@ Q_SIGNALS:
      * * The user created the applet
      * * The applet was moved in from another containment
      * * The applet got restored at startup
+     * @param applet the applet that has been added
+     * @param geometryHint an hint to pass to the GUI on the location
+     *           and size we prefer for the newly created applet;
+     *           the gui might choose whether to respect or not this hint
      */
-    void appletAdded(Plasma::Applet *applet);
+    void appletAdded(Plasma::Applet *applet, const QRectF &geometryHint);
+
+    /**
+     * This signal is emitted right before appletAdded, it can be used
+     * to do a preliminary setup on the applet before the handlers of appletAdded are executed.
+     * Useful for instance to prepare the GUI for the applet
+     * @param applet the applet that is about to be added
+     * @param geometryHint an hint to pass to the GUI on the location
+     *           and size we prefer for the newly created applet;
+     *           the gui might choose whether to respect or not this hint
+     */
+    void appletAboutToBeAdded(Plasma::Applet *applet, const QRectF &geometryHint);
 
     /**
      * This signal is emitted when an applet is destroyed
      */
     void appletRemoved(Plasma::Applet *applet);
 
+    /**
+     * This signal is emitted right before appletRemoved, it can be used
+     * to do a preliminary setup on the applet before the handlers of appletRemoved are executed.
+     * Useful for instance to prepare or teardown the GUI for the applet
+     */
+    void appletAboutToBeRemoved(Plasma::Applet *applet);
+
     /**
      * This signal is emitted when a new applet is created by the containment.
      * Compared to appletAdded, this gets emitted only when the user explicitly
@@ -220,13 +318,23 @@ Q_SIGNALS:
      * @see appletAdded
      * @since 5.16
      */
-    void appletCreated(Plasma::Applet *applet);
+    void appletCreated(Plasma::Applet *applet, const QRectF &geometryHint);
+
+    /**
+     * Emitted when the list of applets has changed, either added or removed
+     */
+    void appletsChanged();
 
     /**
      * Emitted when the activity id has changed
      */
     void activityChanged(const QString &activity);
 
+    /**
+     * Emitted when the activity name has changed
+     */
+    void activityNameChanged(const QString &name);
+
     /**
      * Emitted when the containment requests an add widgets dialog is shown.
      * Usually only used for desktop containments.
@@ -257,7 +365,7 @@ Q_SIGNALS:
     /**
      * Emitted when the wallpaper plugin is changed
      */
-    void wallpaperChanged();
+    void wallpaperPluginChanged();
 
     /**
      * Emitted when the location has changed
@@ -271,6 +379,11 @@ Q_SIGNALS:
      */
     void formFactorChanged(Plasma::Types::FormFactor formFactor);
 
+    /**
+     * Emitted when the containment disaplay hints change
+     */
+    void containmentDisplayHintsChanged(Plasma::Types::ContainmentDisplayHints hints);
+
     /**
      * Emitted when the ui has been fully loaded and is fully working
      * @param uiReady true when the ui of the containment is ready, as well the ui of each applet in it
@@ -282,6 +395,26 @@ Q_SIGNALS:
      */
     void containmentTypeChanged();
 
+    /**
+     * Emitted when the available screen rectangle has changed
+     */
+    void availableRelativeScreenRectChanged(const QRectF &rect);
+
+    /**
+     * Emitted when the available screen rectangle has changed
+     */
+    void availableRelativeScreenRegionChanged(const QList<QRectF> &region);
+
+    /**
+     * Emitted when the screen geometry has changed
+     */
+    void screenGeometryChanged(const QRectF &rect);
+
+    /**
+     * Emitted when the root wallpaper item has changed
+     */
+    void wallpaperGraphicsObjectChanged();
+
 public Q_SLOTS:
     /**
      * Informs the Corona as to what position it is in. This is informational
@@ -300,9 +433,12 @@ public Q_SLOTS:
     void setFormFactor(Plasma::Types::FormFactor formFactor);
 
     /**
-     * Sets the type of this containment.
+     * Set Display hints that come from the containment that suggest the applet how to look and behave.
+     *
+     * @param hints the new hints, as bitwise OR
+     * @since 5.77
      */
-    void setContainmentType(Plasma::Types::ContainmentType type);
+    void setContainmentDisplayHints(Plasma::Types::ContainmentDisplayHints hints);
 
     void reactToScreenChange();
 
@@ -330,7 +466,13 @@ private:
      * @param parent a QObject parent; you probably want to pass in 0
      * @since 4.3
      */
-    Containment(const QString &packagePath, uint appletId);
+    Containment(const KPluginMetaData &md, uint appletId);
+
+    /**
+     * @internal Return root wallpaper item
+     */
+    QObject *wallpaperGraphicsObject() const;
+    void setWallpaperGraphicsObject(QObject *object);
 
     Q_PRIVATE_SLOT(d, void appletDeleted(Plasma::Applet *))
     Q_PRIVATE_SLOT(d, void triggerShowAddWidgets())
