@@ -1,48 +1,35 @@
 /*
- *   Copyright 2013 Marco Martin <mart@kde.org>
- *   Copyright 2015 Eike Hein <hein@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2013 Marco Martin <mart@kde.org>
+    SPDX-FileCopyrightText: 2015 Eike Hein <hein@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
-#include "private/configcategory_p.h"
-#include "configview.h"
 #include "configmodel.h"
 #include "Plasma/Applet"
 #include "Plasma/Containment"
-//#include "plasmoid/wallpaperinterface.h"
-#include "kdeclarative/configpropertymap.h"
+#include "configview.h"
+#include "private/configcategory_p.h"
+#include "sharedqmlengine.h"
 
 #include <QDebug>
 #include <QDir>
 #include <QQmlComponent>
-#include <QQmlEngine>
 #include <QQmlContext>
+#include <QQmlEngine>
 #include <QQuickItem>
 
-#include <klocalizedstring.h>
-#include <kdeclarative/kdeclarative.h>
-#include <KQuickAddons/ConfigModule>
+#include <KLocalizedString>
+#include <KPackage/Package>
+#include <KQuickConfigModule>
+#include <KQuickConfigModuleLoader>
 
 #include <Plasma/Corona>
 #include <Plasma/PluginLoader>
+#include <kquickconfigmoduleloader.h>
 
 namespace PlasmaQuick
 {
-
 //////////////////////////////ConfigModel
 
 class ConfigModelPrivate
@@ -53,16 +40,18 @@ public:
 
     ConfigModel *q;
     QList<ConfigCategory *> categories;
-    QWeakPointer<Plasma::Applet> appletInterface;
-    QHash<QString, KQuickAddons::ConfigModule *> kcms;
+    QPointer<Plasma::Applet> appletInterface;
+    QHash<QString, KQuickConfigModule *> kcms;
 
     void appendCategory(ConfigCategory *c);
+    void removeCategory(ConfigCategory *c);
+    void removeCategoryAt(int index);
     void clear();
     QVariant get(int row) const;
 
-    static ConfigCategory *categories_at(QQmlListProperty<ConfigCategory> *prop, int index);
+    static ConfigCategory *categories_at(QQmlListProperty<ConfigCategory> *prop, qsizetype index);
+    static qsizetype categories_count(QQmlListProperty<ConfigCategory> *prop);
     static void categories_append(QQmlListProperty<ConfigCategory> *prop, ConfigCategory *o);
-    static int categories_count(QQmlListProperty<ConfigCategory> *prop);
     static void categories_clear(QQmlListProperty<ConfigCategory> *prop);
 };
 
@@ -75,12 +64,12 @@ ConfigModelPrivate::~ConfigModelPrivate()
 {
 }
 
-ConfigCategory *ConfigModelPrivate::categories_at(QQmlListProperty<ConfigCategory> *prop, int index)
+ConfigCategory *ConfigModelPrivate::categories_at(QQmlListProperty<ConfigCategory> *prop, qsizetype index)
 {
     ConfigModel *model = qobject_cast<ConfigModel *>(prop->object);
     if (!model || index >= model->d->categories.count() || index < 0) {
-        return 0;
-    }  else {
+        return nullptr;
+    } else {
         return model->d->categories.at(index);
     }
 }
@@ -93,14 +82,14 @@ void ConfigModelPrivate::categories_append(QQmlListProperty<ConfigCategory> *pro
     }
 
     if (o->parent() == prop->object) {
-        o->setParent(0);
+        o->setParent(nullptr);
     }
 
     o->setParent(prop->object);
     model->d->appendCategory(o);
 }
 
-int ConfigModelPrivate::categories_count(QQmlListProperty<ConfigCategory> *prop)
+qsizetype ConfigModelPrivate::categories_count(QQmlListProperty<ConfigCategory> *prop)
 {
     ConfigModel *model = qobject_cast<ConfigModel *>(prop->object);
     if (model) {
@@ -124,19 +113,63 @@ void ConfigModelPrivate::clear()
 {
     q->beginResetModel();
     while (!categories.isEmpty()) {
-        categories.first()->setParent(0);
+        categories.first()->setParent(nullptr);
         categories.pop_front();
     }
     q->endResetModel();
-    emit q->countChanged();
+    Q_EMIT q->countChanged();
 }
 
 void ConfigModelPrivate::appendCategory(ConfigCategory *c)
 {
+    if (!c) {
+        return;
+    }
+
     q->beginInsertRows(QModelIndex(), categories.size(), categories.size());
     categories.append(c);
+
+    auto emitChange = [this, c] {
+        const int row = categories.indexOf(c);
+        if (row > -1) {
+            QModelIndex modelIndex = q->index(row);
+            Q_EMIT q->dataChanged(modelIndex, modelIndex);
+        }
+    };
+
+    QObject::connect(c, &ConfigCategory::nameChanged, q, emitChange);
+    QObject::connect(c, &ConfigCategory::iconChanged, q, emitChange);
+    QObject::connect(c, &ConfigCategory::sourceChanged, q, emitChange);
+    QObject::connect(c, &ConfigCategory::pluginNameChanged, q, emitChange);
+    QObject::connect(c, &ConfigCategory::visibleChanged, q, emitChange);
+
     q->endInsertRows();
-    emit q->countChanged();
+    Q_EMIT q->countChanged();
+}
+
+void ConfigModelPrivate::removeCategory(ConfigCategory *c)
+{
+    const int index = categories.indexOf(c);
+    if (index > -1) {
+        removeCategoryAt(index);
+    }
+}
+
+void ConfigModelPrivate::removeCategoryAt(int index)
+{
+    if (index < 0 || index >= categories.count()) {
+        return;
+    }
+
+    q->beginRemoveRows(QModelIndex(), index, index);
+
+    ConfigCategory *c = categories.takeAt(index);
+    if (c->parent() == q) {
+        c->deleteLater();
+    }
+
+    q->endRemoveRows();
+    Q_EMIT q->countChanged();
 }
 
 QVariant ConfigModelPrivate::get(int row) const
@@ -149,11 +182,8 @@ QVariant ConfigModelPrivate::get(int row) const
     value[QStringLiteral("name")] = categories.at(row)->name();
     value[QStringLiteral("icon")] = categories.at(row)->icon();
     value[QStringLiteral("pluginName")] = categories.at(row)->pluginName();
-    if (appletInterface) {
-        value[QStringLiteral("source")] = QUrl::fromLocalFile(appletInterface.data()->package().filePath("ui", categories.at(row)->source()));
-    } else {
-        value[QStringLiteral("source")] = categories.at(row)->source();
-    }
+    value[QStringLiteral("source")] = q->data(q->index(row, 0), ConfigModel::SourceRole);
+    value[QStringLiteral("includeMargins")] = categories.at(row)->includeMargins();
     value[QStringLiteral("visible")] = categories.at(row)->visible();
     value[QStringLiteral("kcm")] = q->data(q->index(row, 0), ConfigModel::KCMRole);
 
@@ -161,18 +191,9 @@ QVariant ConfigModelPrivate::get(int row) const
 }
 
 ConfigModel::ConfigModel(QObject *parent)
-    : QAbstractListModel(parent),
-      d(new ConfigModelPrivate(this))
+    : QAbstractListModel(parent)
+    , d(new ConfigModelPrivate(this))
 {
-    QHash<int, QByteArray> roleNames;
-    roleNames[NameRole] = "name";
-    roleNames[IconRole] = "icon";
-    roleNames[SourceRole] = "source";
-    roleNames[PluginNameRole] = "pluginName";
-    roleNames[VisibleRole] = "visible";
-    roleNames[KCMRole] = "kcm";
-
-    setRoleNames(roleNames);
 }
 
 ConfigModel::~ConfigModel()
@@ -198,43 +219,48 @@ QVariant ConfigModel::data(const QModelIndex &index, int role) const
         return d->categories.at(index.row())->name();
     case IconRole:
         return d->categories.at(index.row())->icon();
-    case SourceRole:
-    {
+    case SourceRole: {
         const QString source = d->categories.at(index.row())->source();
         // Quick check if source is an absolute path or not
-        if (d->appletInterface && !(source.startsWith('/') && source.endsWith(QLatin1String("qml")))) {
-            return QUrl::fromLocalFile(d->appletInterface.data()->package().filePath("ui", source));
+        if (d->appletInterface && !source.isEmpty() && !(source.startsWith(QLatin1Char('/')) && source.endsWith(QLatin1String("qml")))) {
+            if (!d->appletInterface.data()->kPackage().isValid()) {
+                qWarning() << "wrong applet" << d->appletInterface.data()->pluginMetaData().name();
+            }
+            return d->appletInterface.data()->kPackage().fileUrl("ui", source);
         } else {
             return source;
         }
     }
     case PluginNameRole:
         return d->categories.at(index.row())->pluginName();
+    case IncludeMarginsRole:
+        return d->categories.at(index.row())->includeMargins();
     case VisibleRole:
         return d->categories.at(index.row())->visible();
     case KCMRole: {
         const QString pluginName = d->categories.at(index.row())->pluginName();
-        const QString pluginPath = KPluginLoader::findPlugin(pluginName);
-        //no kcm is registered for this row, it's a normal qml-only entry
-        if (pluginName.isEmpty() || pluginPath.isEmpty()) {
+        // no kcm is registered for this row, it's a normal qml-only entry
+        if (pluginName.isEmpty()) {
             return QVariant();
         }
 
         if (d->kcms.contains(pluginName)) {
             return QVariant::fromValue(d->kcms.value(pluginName));
         }
-
-        KPluginLoader loader(pluginPath);
-        KPluginFactory* factory = loader.factory();
-        if (!factory) {
-            qWarning() << "Error loading KCM:" << loader.errorString();
-        } else {
-            KQuickAddons::ConfigModule *cm = factory->create<KQuickAddons::ConfigModule >(const_cast<ConfigModel *>(this));
-            if (!cm) {
-                qWarning() << "Error creating KCM object from plugin" << loader.fileName();
+        auto parent = const_cast<ConfigModel *>(this);
+        auto engine = new PlasmaQuick::SharedQmlEngine(parent);
+        auto cmResult = KQuickConfigModuleLoader::loadModule(KPluginMetaData(pluginName), parent, QVariantList(), engine->engine());
+        if (KQuickConfigModule *cm = cmResult.plugin) {
+            if (QQmlContext *ctx = QQmlEngine::contextForObject(this)) {
+                // assign the ConfigModule the same QML context as we have so it can use the same QML engine as we do
+                QQmlEngine::setContextForObject(cmResult.plugin, ctx);
             }
+
             d->kcms[pluginName] = cm;
             return QVariant::fromValue(cm);
+        } else {
+            qWarning() << "Error loading KCM:" << cmResult.errorText;
+            return QVariant();
         }
     }
     default:
@@ -242,13 +268,25 @@ QVariant ConfigModel::data(const QModelIndex &index, int role) const
     }
 }
 
+QHash<int, QByteArray> ConfigModel::roleNames() const
+{
+    return {
+        {NameRole, "name"},
+        {IconRole, "icon"},
+        {SourceRole, "source"},
+        {PluginNameRole, "pluginName"},
+        {IncludeMarginsRole, "includeMargins"},
+        {VisibleRole, "visible"},
+        {KCMRole, "kcm"},
+    };
+}
+
 QVariant ConfigModel::get(int row) const
 {
     return d->get(row);
 }
 
-void ConfigModel::appendCategory(const QString &iconName, const QString &name,
-                                 const QString &path, const QString &pluginName)
+void ConfigModel::appendCategory(const QString &iconName, const QString &name, const QString &path, const QString &pluginName)
 {
     ConfigCategory *cat = new ConfigCategory(this);
     cat->setIcon(iconName);
@@ -258,8 +296,7 @@ void ConfigModel::appendCategory(const QString &iconName, const QString &name,
     d->appendCategory(cat);
 }
 
-void ConfigModel::appendCategory(const QString &iconName, const QString &name,
-                                 const QString &path, const QString &pluginName, bool visible)
+void ConfigModel::appendCategory(const QString &iconName, const QString &name, const QString &path, const QString &pluginName, bool visible)
 {
     ConfigCategory *cat = new ConfigCategory(this);
     cat->setIcon(iconName);
@@ -270,6 +307,21 @@ void ConfigModel::appendCategory(const QString &iconName, const QString &name,
     d->appendCategory(cat);
 }
 
+void ConfigModel::appendCategory(ConfigCategory *category)
+{
+    d->appendCategory(category);
+}
+
+void ConfigModel::removeCategory(ConfigCategory *category)
+{
+    d->removeCategory(category);
+}
+
+void ConfigModel::removeCategoryAt(int index)
+{
+    d->removeCategoryAt(index);
+}
+
 void ConfigModel::clear()
 {
     d->clear();
@@ -287,11 +339,12 @@ Plasma::Applet *ConfigModel::applet() const
 
 QQmlListProperty<ConfigCategory> ConfigModel::categories()
 {
-    return QQmlListProperty<ConfigCategory>(this, 0, ConfigModelPrivate::categories_append,
+    return QQmlListProperty<ConfigCategory>(this,
+                                            nullptr,
+                                            ConfigModelPrivate::categories_append,
                                             ConfigModelPrivate::categories_count,
                                             ConfigModelPrivate::categories_at,
                                             ConfigModelPrivate::categories_clear);
-
 }
 
 }
